@@ -1,11 +1,19 @@
-use pyo3::exceptions::PyException;
+use pyo3::create_exception;
 use pyo3::prelude::*;
 use pyo3::PyErr;
 
 use std::{convert, fmt};
 
-// TODO: get this type exposed to Python as an exception (for now using Exception)
+pub type Result<T> = std::result::Result<T, SignalProtocolError>;
+
+create_exception!(
+    error,
+    SignalProtocolException,
+    pyo3::exceptions::PyException
+);
+
 #[pyclass]
+#[derive(Debug)]
 pub struct SignalProtocolError {
     pub err: libsignal_protocol_rust::SignalProtocolError,
 }
@@ -18,7 +26,7 @@ impl fmt::Display for SignalProtocolError {
 
 impl convert::From<SignalProtocolError> for PyErr {
     fn from(err: SignalProtocolError) -> Self {
-        PyException::new_err(err.to_string())
+        SignalProtocolException::new_err(err.to_string())
     }
 }
 
@@ -33,13 +41,20 @@ impl SignalProtocolError {
         Self { err }
     }
 
+    pub fn err_from_str(err: String) -> PyErr {
+        SignalProtocolException::new_err(err)
+    }
+
     pub fn new_err(err: libsignal_protocol_rust::SignalProtocolError) -> PyErr {
         let local_error = SignalProtocolError { err };
-        PyException::new_err(local_error.to_string())
+        SignalProtocolException::new_err(local_error.to_string())
     }
 }
 
-pub fn init_submodule(_py: Python, module: &PyModule) -> PyResult<()> {
-    module.add_class::<SignalProtocolError>()?;
+pub fn init_submodule(py: Python, module: &PyModule) -> PyResult<()> {
+    module.add(
+        "SignalProtocolException",
+        py.get_type::<SignalProtocolException>(),
+    )?;
     Ok(())
 }
